'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _shockedCommon = require('shocked-common');

var _CloseError = require('./CloseError');

var _CloseError2 = _interopRequireDefault(_CloseError);

var _TimeoutError = require('./TimeoutError');

var _TimeoutError2 = _interopRequireDefault(_TimeoutError);

var _initStore = require('./initStore');

var _batchActions = require('./batchActions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultOptions = {
  timeout: 30000
};

var TrackerClient = function (_EventEmitter) {
  _inherits(TrackerClient, _EventEmitter);

  function TrackerClient(store, group, params, destroyer) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : defaultOptions;

    _classCallCheck(this, TrackerClient);

    var _this = _possibleConstructorReturn(this, (TrackerClient.__proto__ || Object.getPrototypeOf(TrackerClient)).call(this));

    _this.store = store;
    _this.group = group;
    _this.params = params;
    _this.destroy = destroyer;
    _this.apis = null;
    _this.serial = 0;
    _this.options = options;

    _this.sn = 0;
    _this.calls = [];
    return _this;
  }

  _createClass(TrackerClient, [{
    key: 'onConnect',
    value: function onConnect(client) {
      this.client = client;
      client.send((0, _shockedCommon.PKT_TRACKER_CREATE)(this.group, this.params, this.serial));
    }
  }, {
    key: 'onDisconnect',
    value: function onDisconnect() {
      this.client = null;

      // The api calls are not expected to be completed now
      // reject them with an error
      if (this.calls.length > 0) {
        var calls = this.calls;

        this.calls = [];

        var err = new _CloseError2.default('Connection Terminated');
        calls.forEach(function (_ref) {
          var reject = _ref.reject;
          return reject(err);
        });
      }
    }
  }, {
    key: 'onCreate',
    value: function onCreate(serial, data, apis) {
      var _this2 = this;

      this.serial = serial;

      // Generate api as soon as the tracker is opened
      this.apis = apis.map(function (name) {
        return _this2.createApi(name);
      });

      this.store.dispatch((0, _initStore.initStore)(data));
    }
  }, {
    key: 'onUpdate',
    value: function onUpdate(serial, actions) {
      this.serial = serial;

      this.store.dispatch((0, _batchActions.batchActions)(actions));
    }
  }, {
    key: 'createApi',
    value: function createApi(name) {
      var _this3 = this;

      return function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return new Promise(function (resolve, reject) {
          _this3.sn += 1;
          var callId = _this3.sn;
          var pkt = (0, _shockedCommon.PKT_TRACKER_API)(_this3.group, callId, name, args);
          _this3.client.send(pkt);

          // Setup a timeout, to cleanup in case a responsee is not
          // received after a long time.
          var timeout = _this3.options.timeout && setTimeout(function () {
            _this3.finishApi(callId, false, new _TimeoutError2.default('Api call ' + name + '[' + _this3.callId + '] didn\'t complete within ' + _this3.options.timeout + ' ms'));
          }, _this3.options.timeout);

          // Remember the call
          _this3.calls.push({
            callId: callId, resolve: resolve, reject: reject, timeout: timeout
          });
        });
      };
    }
  }, {
    key: 'finishApi',
    value: function finishApi(id, status, response) {
      var idx = this.calls.findIndex(function (c) {
        return c.callId === id;
      });
      if (idx >= 0) {
        var call = this.calls.splice(idx, 1)[0];
        if (status) {
          call.resolve(response);
        } else {
          call.reject(response);
        }
        return true;
      }

      return false;
    }
  }, {
    key: 'onApiResponse',
    value: function onApiResponse(callId, status, response, params) {
      Object.assign(this.params, params);
      this.finishApi(callId, status, response);
    }
  }, {
    key: 'onAction',
    value: function onAction(action) {
      this.serial = action.$serial$;
      this.store.dispatch(action);
    }
  }, {
    key: 'getApi',
    value: function getApi() {
      return this.apis;
    }
  }, {
    key: 'close',
    value: function close() {
      // Make sure the tracker is fully destroyed
      this.destroy();

      // Gracefully close the tracker, if the client is
      // connected;
      if (this.client) {
        this.client.send((0, _shockedCommon.PKT_TRACKER_CLOSE)(this.group));
      }
    }
  }]);

  return TrackerClient;
}(_events2.default);

exports.default = TrackerClient;